@@ -8,5 +8,7 @@ from .softmax2 import *
 from .vit import *
 from .depthwise_convolution import *
 from .convolution import *
+from .separable_convolution import *
 from .reshaping import *
 from .attention import *
+from .pooling import *
