@@ -18,45 +18,15 @@ import numpy as np
 import tensorflow as tf
 import keras
 
-from .layers import deserialize_quant_object
+from .layers import deserialize_quant_object, Calibrable
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 
 
 __all__ = ["QuantizedGELU", "QuantizedReLU"]
 
 
-def act_to_qlayer(act, config):
-    """Get quantized activation layer from string.
-
-    This is needed because one cannot serialize a class in layer.get_config, the string is thus
-    serialized instead.
-
-    Args:
-        act (str): string that values in ['GeLU', 'ReLUx] and that allows to
-            choose activation inside MLP.
-        config (str): quantization configuration
-
-    Returns:
-        tf.keras.layers: the activation layer class
-    """
-    if act == "GeLU":
-        act_funct = QuantizedGELU(quant_config=config)
-    elif act[:4] == 'ReLU':
-        try:
-            max_value = int(act[4:])
-        except ValueError:
-            raise ValueError(
-                "ReLU must be in the form 'ReLUx', where x is the max-value")
-        act_funct = QuantizedReLU(quant_config=config, max_value=max_value)
-    else:
-        raise NotImplementedError(
-            f"act should be in ['GeLU', 'ReLUx'] but received {act}.")
-
-    return act_funct
-
-
 @tf.keras.utils.register_keras_serializable()
-class QuantizedGELU(keras.layers.Layer):
+class QuantizedGELU(Calibrable, keras.layers.Layer):
     """Quantize the `Gaussian Error Linear Units (GELU)<https://arxiv.org/abs/1606.08415>`_
     activation, following the implementation on `i-BERT <https://arxiv.org/abs/2101.01321>`_.
 
@@ -82,7 +52,7 @@ class QuantizedGELU(keras.layers.Layer):
     :math:`8.2\times10^{-3} / 1.8\times10^{-2}` (respectively), within the range :math:`|x| < 1`.
 
     Args:
-        quant_config (dict): quantization configuration of input/ouput quantizers.
+        quant_config (dict): quantization configuration of input/output quantizers.
     """
     unsupported_args = {
         'approximate': True}
@@ -103,7 +73,7 @@ class QuantizedGELU(keras.layers.Layer):
         self._b_int = FixedPoint.quantize(-1.769, frac_bits, value_bits)
         self._c_int = FixedPoint.quantize(1.0, frac_bits, value_bits)
         # Coefficients used in i_gelu function
-        self._s2_int = FixedPoint.quantize(1.0/1.4142, frac_bits, value_bits)
+        self._s2_int = FixedPoint.quantize(1.0 / 1.4142, frac_bits, value_bits)
         self._05_int = FixedPoint.quantize(0.5, frac_bits, value_bits)
         self.out_quantizer = deserialize_quant_object(
             self.quant_config, "output_quantizer", False)
@@ -133,7 +103,7 @@ class QuantizedGELU(keras.layers.Layer):
         Note:
             ``S`` denotes the quantization scale.
         """
-        return (q_in + q_b)**2*q_a + q_c
+        return (q_in + q_b)**2 * q_a + q_c
 
     def i_erf(self, q_in):
         """Approximate the error function with a degree two polynomial.
@@ -162,7 +132,7 @@ class QuantizedGELU(keras.layers.Layer):
         # Get coefficients needed in i_gelu function
         q_s2, q_05 = self._gelu_coeff
         # Compute i_erf function
-        q_erf = self.i_erf(q_in*q_s2)
+        q_erf = self.i_erf(q_in * q_s2)
         # Compute i-GELU function
         return (q_erf + 1) * q_05 * q_in
 
@@ -201,7 +171,7 @@ class QuantizedGELU(keras.layers.Layer):
 
 
 @tf.keras.utils.register_keras_serializable()
-class QuantizedReLU(keras.layers.Layer):
+class QuantizedReLU(Calibrable, keras.layers.Layer):
     """Quantized version of the ReLU activation layer applicable on FixedPoint tensor.
     """
     unsupported_args = {
@@ -211,6 +181,11 @@ class QuantizedReLU(keras.layers.Layer):
     def __init__(self, *args, max_value=6, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
         self.quant_config = quant_config
+
+        if max_value is None:
+            raise ValueError(
+                f"QuantizedRelU layer {self.name} has been initialized with\
+                    unsupported None max_value.")
         if isinstance(max_value, np.ndarray):
             self.max_value = int(max_value.item())
         else:
