@@ -23,14 +23,14 @@ from keras import backend
 from keras.layers import DepthwiseConv2D
 
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
-from .layers import deserialize_quant_object
+from .layers import deserialize_quant_object, Calibrable
 
 
 __all__ = ["QuantizedDepthwiseConv2D"]
 
 
 @tf.keras.utils.register_keras_serializable()
-class QuantizedDepthwiseConv2D(DepthwiseConv2D):
+class QuantizedDepthwiseConv2D(Calibrable, DepthwiseConv2D):
     """ A depthwise convolutional layer that operates on quantized inputs and weights.
     """
 
