@@ -18,7 +18,7 @@ import tensorflow as tf
 import keras
 from tensorflow.python.framework import ops
 
-from .layers import deserialize_quant_object
+from .layers import deserialize_quant_object, Calibrable
 from ..tensors import FixedPoint, reciprocal_lut, MAX_BUFFER_BITWIDTH
 
 
@@ -66,7 +66,7 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
             mean = tf.math.reduce_mean(
                 inputs, axes, *args, **kwargs, name="mean")
             mad = tf.math.reduce_mean(tf.math.abs(
-                inputs-mean), axes, *args, **kwargs, name="mad")
+                inputs - mean), axes, *args, **kwargs, name="mad")
         return mean, mad ** 2
 
     def call(self, inputs, training=None):
@@ -112,7 +112,7 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
 
 
 @tf.keras.utils.register_keras_serializable()
-class QuantizedLayerNormalization(LayerMadNormalization):
+class QuantizedLayerNormalization(Calibrable, LayerMadNormalization):
 
     def __init__(self, *args, quant_config={}, **kwargs):
         super().__init__(*args, **kwargs)
