@@ -18,9 +18,9 @@ from contextlib import contextmanager
 import tensorflow as tf
 import keras.backend as K
 from keras.layers import Layer
-from keras.utils import control_flow_util
 
 from ..tensors import QTensor, QFloat, FixedPoint
+from .layers import Calibrable
 
 
 @contextmanager
@@ -36,7 +36,7 @@ def disable_partitioner(layer):
             layer._scope.set_partitioner(partitioner)
 
 
-class Quantizer(Layer):
+class Quantizer(Calibrable, Layer):
     """The base class for all quantizers.
 
     The bitwidth defines the number of quantization levels on which the
@@ -163,26 +163,6 @@ class Quantizer(Layer):
                 with tf.compat.v1.colocate_with(variable):
                     return tf.compat.v1.assign(variable, value_r, name=scope)
 
-    def _get_training_value(self, training=None):
-        """Read training mode from the context. Function taken of
-        `BatchNormalizationLayer <https://bit.ly/3xNYfxg>`_ code.
-
-        Args:
-            training (bool, optional): the training mode. Defaults to None.
-
-        Returns:
-            bool: the training mode: True if training, False if inference.
-        """
-        if training is None:
-            training = K.learning_phase()
-        if self._USE_V2_BEHAVIOR:
-            if isinstance(training, int):
-                training = bool(training)
-            if not self.trainable:
-                # When the layer is not trainable, it overrides the value passed from model
-                training = False
-        return control_flow_util.constant_value(training)
-
     @staticmethod
     def _update_weights(inputs, variable, axis, input_batch_size=None, momentum=0.9):
         """Given the inputs tensor, a variable is updated with the maximum reduction of
@@ -220,9 +200,9 @@ class Quantizer(Layer):
 
     def call(self, inputs, training=None):
         """Update the weights from moving average, and quantize the inputs in three steps:
-            1. Get information of training mode
-            2. Update the weights from moving average
-            3. Quantize the inputs
+            1. Convert the inputs to float values,
+            2. Update the weights from moving average, only if calibration is enabled and
+            3. Quantize the inputs.
 
         Args:
             inputs (:obj:`tensorflow.Tensor` or :obj:`QTensor`): the inputs tensor.
@@ -233,8 +213,8 @@ class Quantizer(Layer):
         """
         if isinstance(inputs, QTensor):
             inputs = inputs.to_float()
-        training = self._get_training_value(training)
-        if training:  # Retrieve information from the inputs and update the weights
+        if self.calibration:
+            # Retrieve information from the inputs and update the weights
             input_batch_size = tf.shape(inputs)[0]
             if tf.reduce_all(tf.math.equal(self.max_value, tf.constant(1.))):
                 momentum = tf.constant(-1.)
@@ -265,17 +245,15 @@ class UniformQuantizer(Quantizer):
 
     Args:
         bitwidth (int): the quantization bitwidth.
-        signed (bool, optional): whether the quantizer expects signed values or unsigned.
-            Defaults to None.
         scale_bits (int, optional): the number of bits for the scaling. Defaults to 24.
     """
 
-    def __init__(self, bitwidth, signed=True, scale_bits=24, **kwargs):
+    def __init__(self, bitwidth, scale_bits=24, **kwargs):
         if not isinstance(bitwidth, int) or bitwidth < 2:
             raise ValueError(
                 f"Bitwidth should be an int >= 2, currently {bitwidth}")
         self.scale_bits = scale_bits
-        super().__init__(bitwidth, signed, **kwargs)
+        super().__init__(bitwidth, **kwargs)
 
     def quantize(self, x):
         """Quantize float inputs
@@ -337,3 +315,27 @@ class FixedPointQuantizer(Quantizer):
         if not self.signed:
             x = tf.math.maximum(x, 0)
         return FixedPoint.quantize(x, frac_bits, self.value_bits)
+
+
+@tf.keras.utils.register_keras_serializable()
+class Dequantizer(Layer):
+
+    def call(self, inputs, training=None):
+        """Convert QTensor inputs to float.
+
+        Args:
+            inputs (:obj:`tensorflow.Tensor` or :obj:`QTensor`): the inputs tensor(s).
+            training (bool, optional): the training mode. Defaults to None.
+
+        Returns:
+            :obj:`tensorflow.Tensor`: the quantized tensor(s).
+        """
+        def dequantize(x):
+            if isinstance(x, QTensor):
+                return x.to_float()
+            return x
+
+        if isinstance(inputs, (list, tuple)):
+            return [dequantize(x) for x in inputs]
+
+        return dequantize(inputs)
