@@ -14,11 +14,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
-
+import keras
 import tensorflow as tf
 
-from .layers import deserialize_quant_object
-from ..tensors import QTensor, FixedPoint, MAX_BUFFER_BITWIDTH
+from .layers import deserialize_quant_object, Calibrable
+from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 
 
 # Restrict list of exported symbols on default import
@@ -27,12 +27,12 @@ __all__ = ["ClassToken", "QuantizedClassToken",
 
 
 @tf.keras.utils.register_keras_serializable()
-class ClassToken(tf.keras.layers.Layer):
+class ClassToken(keras.layers.Layer):
     """Append a class token to an input layer."""
 
     def build(self, input_shape):
         super().build(input_shape)
-        cls_init = tf.keras.initializers.TruncatedNormal(stddev=0.02)
+        cls_init = keras.initializers.TruncatedNormal(stddev=0.02)
         self.hidden_size = input_shape[-1]
         self.cls = tf.Variable(
             name="cls",
@@ -51,14 +51,14 @@ class ClassToken(tf.keras.layers.Layer):
 
 
 @tf.keras.utils.register_keras_serializable()
-class AddPositionEmbs(tf.keras.layers.Layer):
+class AddPositionEmbs(keras.layers.Layer):
     """Adds (optionally learned) positional embeddings to the inputs."""
 
     def build(self, input_shape):
         assert len(
             input_shape) == 3, f"Number of dimensions should be 3, got {len(input_shape)}"
         super().build(input_shape)
-        pe_init = tf.keras.initializers.TruncatedNormal(stddev=0.02)
+        pe_init = keras.initializers.TruncatedNormal(stddev=0.02)
         self.pe = tf.Variable(
             name="pos_embedding",
             initial_value=pe_init(shape=(1, input_shape[1], input_shape[2])),
@@ -71,7 +71,7 @@ class AddPositionEmbs(tf.keras.layers.Layer):
 
 
 @tf.keras.utils.register_keras_serializable()
-class QuantizedClassToken(ClassToken):
+class QuantizedClassToken(Calibrable, ClassToken):
     """Quantize the :class:`ClassToken` layer, allowing quantization of the output.
     """
 
@@ -80,11 +80,31 @@ class QuantizedClassToken(ClassToken):
         self.quant_config = quant_config
         self.out_quantizer = deserialize_quant_object(
             self.quant_config, "output_quantizer", False)
+        self.cls_quantizer = deserialize_quant_object(
+            self.quant_config, "cls_quantizer", True)
+        self.buffer_bitwidth = self.quant_config.get(
+            "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
 
     def call(self, inputs, training=None):
-        if isinstance(inputs, QTensor):
-            inputs = inputs.to_float()
-        outputs = super().call(inputs, training)
+        # raise an error if the inputs are not FixedPoint
+        if not isinstance(inputs, FixedPoint):
+            raise TypeError(f"QuantizedClassToken only accepts FixedPoint\
+                              inputs. Receives {type(inputs)} inputs.")
+
+        batch_size = tf.shape(inputs.values)[0]
+        inputs = inputs.promote(self.buffer_bitwidth)
+
+        cls = self.cls_quantizer(self.cls, training=training)
+        cls = cls.promote(self.buffer_bitwidth)
+
+        cls_broadcasted = tf.broadcast_to(cls, [batch_size, 1, self.hidden_size])
+
+        # Align inputs and cls_brodcasted frac_bits values before concatenation
+        inputs = inputs.align(cls_broadcasted)[0]
+        cls_broadcasted = cls_broadcasted.align(inputs)[0]
+
+        outputs = tf.concat([cls_broadcasted, inputs], 1)
+
         if self.out_quantizer is not None:
             outputs = self.out_quantizer(outputs, training=training)
         return outputs
@@ -96,7 +116,7 @@ class QuantizedClassToken(ClassToken):
 
 
 @tf.keras.utils.register_keras_serializable()
-class QuantizedAddPositionEmbs(AddPositionEmbs):
+class QuantizedAddPositionEmbs(Calibrable, AddPositionEmbs):
     """Quantize the :class:`AddPositionEmbs` layer, allowing operations in FixedPoint domain.
     """
 
