@@ -107,6 +107,34 @@ class FixedPoint(QTensor):
                 "Reducing the number of value bits is lossy: use a quantizer instead")
         return FixedPoint(self.values, self.frac_bits, bits)
 
+    def align(self, other=None):
+        """Align fractional bits
+
+        This returns an equivalent FixedPoint with a scalar fractional bit
+        corresponding to the maximum of:
+            - the initial fractional bits on all channels,
+            - when specified, the other FixedPoint fractional bits on all channels.
+
+        This is required before performing an operation that adds or subtracts
+        elements along the last dimension, to make sure all these elements are
+        in the same scale.
+
+        Args:
+            other(:obj:`FixedPoint`): a FixedPoint to align to
+
+        Returns:
+            tuple(:obj:`FixedPoint`, :obj:`tf.Tensor`): a new FixedPoint with aligned
+                fractional bits and the shift that was applied.
+        """
+        max_frac_bits = tf.reduce_max(self.frac_bits)
+        if other is not None:
+            if not isinstance(other, FixedPoint):
+                raise ValueError("Other must be a FixedPoint")
+            max_frac_bits = tf.math.maximum(max_frac_bits, tf.reduce_max(other.frac_bits))
+        shift = max_frac_bits - self.frac_bits
+        values = FixedPoint._lshift(self.values, shift)
+        return FixedPoint(values, max_frac_bits, self.value_bits), shift
+
     @staticmethod
     def _rshift(values, shift):
         return floor_through(values / FixedPoint._pow2(shift))
