@@ -17,7 +17,7 @@
 import warnings
 import tensorflow as tf
 import keras
-from typing import Union
+from typing import List, Union
 
 from .qtensor import QTensor, floor_through
 from .fixed_point import FixedPoint
@@ -32,6 +32,21 @@ def warn_float_conversion(x, op):
     warnings.warn(f"Reverting {x_name} to float to perform {op} with Tensor")
 
 
+def assert_fp_per_tensor(x):
+    tf.debugging.assert_rank(x.frac_bits, tf.constant(0))
+
+
+def assert_qf_per_tensor(x):
+    tf.debugging.assert_rank(x.scales.values, tf.constant(0))
+
+
+def assert_per_tensor(x):
+    if isinstance(x, FixedPoint):
+        assert_fp_per_tensor(x)
+    elif isinstance(x, QFloat):
+        assert_qf_per_tensor(x)
+
+
 @tf.experimental.dispatch_for_binary_elementwise_apis(FixedPoint, tf.Tensor)
 def fp_tensor_binary_elementwise_api_handler(api_func, x, y):
     warn_float_conversion(x, api_func)
@@ -162,31 +177,20 @@ def fp_matmul(a: FixedPoint,
     tf.debugging.assert_rank_at_least(a.values, 2)
     tf.debugging.assert_rank_at_least(b.values, 2)
 
-    # Evaluate the maximum number of fractional bits
-    a_frac_bits = tf.reduce_max(a.frac_bits)
-
     if not transpose_a:
-        # If a has multiple fractional bits, we must align the values along the
-        # last dimension before the matmul collapses it
-        a = tf.cond(tf.rank(a.frac_bits) > 0,
-                    lambda: a << (a_frac_bits - a.frac_bits),
-                    lambda: a)
+        # Since the last dimension is collapsed by the matmul, a must be quantized per-tensor
+        assert_fp_per_tensor(a)
 
     if isinstance(b, FixedPoint):
         if transpose_b:
-            # Because of the transposition, the quantization dim of b will be
-            # collapsed by matmul, so we might need to align values
-            b_frac_bits = tf.reduce_max(b.frac_bits)
-            b = tf.cond(tf.rank(b.frac_bits) > 0,
-                        lambda: b << (b_frac_bits - b.frac_bits),
-                        lambda: b)
-        else:
-            b_frac_bits = b.frac_bits
+            # Since the last dimension is collapsed by the matmul, b must be quantized per-tensor
+            assert_fp_per_tensor(b)
+        b_frac_bits = b.frac_bits
     else:
         b_frac_bits = b.scales.frac_bits
         if transpose_b:
             # We only support tranposed matmul if the QFloat is per-tensor
-            tf.debugging.assert_rank(tf.rank(b.scales.values), 0)
+            assert_qf_per_tensor(b)
 
     # Do matmul on values
     m_values = floor_through(
@@ -197,7 +201,7 @@ def fp_matmul(a: FixedPoint,
         m_values *= b.scales.values
 
     # Return a new FixedPoint
-    return FixedPoint(m_values, a_frac_bits + b_frac_bits, a.value_bits)
+    return FixedPoint(m_values, a.frac_bits + b_frac_bits, a.value_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.shape)
@@ -207,18 +211,86 @@ def fp_shape(input: FixedPoint, out_type=tf.dtypes.int32, name=None):
 
 @tf.experimental.dispatch_for_api(tf.reshape)
 def fp_reshape(tensor: FixedPoint, shape, name=None):
-    tf.assert_rank(tensor.frac_bits, tf.constant(0))
+    assert_fp_per_tensor(tensor)
     output = tf.reshape(tensor.values, shape, name)
     return FixedPoint(output, tensor.frac_bits, tensor.value_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.transpose)
 def fp_transpose(a: FixedPoint, perm=None, conjugate=False, name="transpose"):
-    tf.assert_rank(a.frac_bits, tf.constant(0))
+    assert_fp_per_tensor(a)
     output = tf.transpose(a.values, perm, conjugate, name)
     return FixedPoint(output, a.frac_bits, a.value_bits)
 
 
+@tf.experimental.dispatch_for_api(tf.broadcast_to)
+def fp_brodcast_to(input: FixedPoint, shape, name=None):
+    """Broadcast a FixedPoint tensor for a compatible shape.
+
+    Args:
+        input (:obj:`FixedPoint`): a FixedPoint to broadcast.
+        shape (:obj:`tf.Tensor`): an 1-D `int` Tensor representing
+            the shape of the desired output. Must be one of the
+            following types: `int32`, `int64`.
+        name (str, optional): a name for the operation. Defaults to None.
+
+    Returns:
+        :obj:`FixedPoint`: the brodcasted output. Has the same
+            type as `input`.
+  """
+    # Check first that the last dimension is unchanged
+    tf.assert_equal(input.shape[-1], shape[-1], message="To brodcast FixedPoint input,\
+                        last dimension should remain unchanged")
+    output = tf.broadcast_to(input.values, shape, name)
+    return FixedPoint(output, input.frac_bits, input.value_bits)
+
+
+@tf.experimental.dispatch_for_api(tf.concat)
+def fp_concat(values: List[FixedPoint], axis, name="concat"):
+    """Concatenates FixedPoint tensors along one dimension.
+
+    Args:
+        values (List of :obj:`FixedPoint`): List of FixedPoint tensors
+            to concatenate.
+        axis (list): Dimension along which to concatenate.
+        name (str, optional): the name for the Tensorflow ops.
+            Defaults to "concat".
+
+    Returns:
+        :obj:`FixedPoint`: the concatenate output FixedPoint.
+    """
+    if len(values) == 1:
+        return FixedPoint(values[0].values, values[0].frac_bits, values[0].value_bits)
+
+    # For now we only support concatenation of one or two elements
+    tf.assert_equal(
+        len(values),
+        2, f"We only support concatenation of one or two FixedPoint. \
+           Receives {len(values)} tensors as input.")
+
+    # For now we don't support concatenation over last dimension
+    rank_input = tf.rank(values[0].values)
+    if axis < 0:
+        dim = axis + rank_input
+    else:
+        dim = axis
+    tf.Assert(tf.math.not_equal(dim, rank_input - 1),
+              ['we do not support concatenation over last axis. axis is:', axis])
+
+    # FixedPoint tensors to concatenate should be per-tensor
+    assert_fp_per_tensor(values[0])
+    assert_fp_per_tensor(values[1])
+
+    tf.assert_equal(values[0].frac_bits, values[1].frac_bits, message=f"the two\
+            FixedPoint must have the same frac_bits. Receives {values[0].frac_bits}\
+            and {values[1].frac_bits}")
+    values_out = tf.concat([values[0].values, values[1].values], axis, name)
+
+    return FixedPoint(
+        values_out, values[0].frac_bits, max(
+            values[0].value_bits, values[1].value_bits))
+
+
 @tf.experimental.dispatch_for_api(tf.math.reduce_max)
 def fp_reduce_max(input_tensor: FixedPoint,
                   axis=None,
@@ -237,44 +309,16 @@ def fp_reduce_max(input_tensor: FixedPoint,
     Returns:
         :obj:`FixedPoint`: the maximum FixedPoint.
     """
-    # The sum fractional bits is the max on each exis
-    if len(input_tensor.frac_bits.shape) == 0:
-        s_frac_bits = tf.math.reduce_max(input_tensor.frac_bits)
-    else:
-        s_frac_bits = tf.math.reduce_max(input_tensor.frac_bits,
-                                         axis=axis,
-                                         keepdims=keepdims)
-    # Align all terms on the same fractional bits: in reality it will only
-    # keep the bigger one
-    input_values = FixedPoint._lshift(input_tensor.values,
-                                      s_frac_bits - input_tensor.frac_bits)
+    # We only support reduce_max if the input is per-tensor
+    assert_fp_per_tensor(input_tensor)
+
     # Reduce max
-    s_values = tf.math.reduce_max(input_values,
+    s_values = tf.math.reduce_max(input_tensor.values,
                                   axis,
                                   keepdims=keepdims,
                                   name=name)
     # Return a new FixedPoint
-    return FixedPoint(s_values, s_frac_bits, input_tensor.value_bits)
-
-
-@tf.experimental.dispatch_for_api(tf.math.maximum)
-def fp_maximum(x: FixedPoint, y: FixedPoint, name=None):
-    """Returns the max of x and y (i.e. x > y ? x : y) element-wise.
-
-    Args:
-        x (:obj:`FixedPoint`): the FixedPoint tensor.
-        y (:obj:`FixedPoint`): the FixedPoint tensor.
-        name (str, optional): the name for the Tensorflow ops. Defaults to None.
-
-    Returns:
-        :obj:`FixedPoint`: s FixedPoint containing maximum values.
-    """
-    # Align all terms on the same fractional bits
-    a_frac_bits, a_values, b_values = x._align_values(y)
-    # Apply maximum
-    max_values = tf.math.maximum(a_values, b_values, name=name)
-    # Return a new FixedPoint
-    return FixedPoint(max_values, a_frac_bits, x.value_bits)
+    return FixedPoint(s_values, input_tensor.frac_bits, input_tensor.value_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.compat.v1.gather)
@@ -293,6 +337,9 @@ def fp_gather(params: Union[FixedPoint, QFloat], indices, validate_indices=None,
     Returns:
         :obj:`QTensor`: a QTensor containing the gathered values.
     """
+    # We do not support gather along the last axis if the QTensor is per-axis
+    if axis in (-1, len(params.shape)):
+        assert_per_tensor(params)
     # Create a new QTensor, gather the indices in a desired axis
     x_gather = tf.gather(params.values, indices, validate_indices, axis, batch_dims, name)
     if isinstance(params, QFloat):
